@@ -82,14 +82,17 @@ public class li_std_vector_runme {
       } catch (ArgumentException) {
       }
     }
+#if SWIG_DOTNET_1
     {
+      // runtime check that 2D arrays fail
       double[,] outputarray = new double[collectionSize,collectionSize];
       try {
         vect.CopyTo(outputarray);
-        throw new Exception("CopyTo (5) test failed");
+        throw new Exception("CopyTo (5a) test failed");
       } catch (ArgumentException) {
       }
     }
+#endif
     {
       StructVector inputvector = new StructVector();
       int arrayLen = 10;
@@ -154,6 +157,9 @@ public class li_std_vector_runme {
         throw new Exception("ICollection constructor null test failed");
       } catch (ArgumentNullException) {
       }
+      {
+        myDoubleVector = new DoubleVector() { 123.4, 567.8, 901.2 };
+      }
 
       // IndexOf() test
       for (int i=0; i<collectionSize; i++) {
@@ -187,11 +193,22 @@ public class li_std_vector_runme {
         throw new Exception("Repeat count test failed");
       
       // Also tests enumerator
-      System.Collections.IEnumerator myEnumerator = dv.GetEnumerator();
-      while ( myEnumerator.MoveNext() ) {
-         if ((double)myEnumerator.Current != 77.7)
-           throw new Exception("Repeat test failed");
+      {
+        System.Collections.IEnumerator myEnumerator = dv.GetEnumerator();
+        while ( myEnumerator.MoveNext() ) {
+           if ((double)myEnumerator.Current != 77.7)
+             throw new Exception("Repeat (1) test failed");
+        }
       }
+#if !SWIG_DOTNET_1
+      {
+        System.Collections.Generic.IEnumerator<double> myEnumerator = dv.GetEnumerator();
+        while ( myEnumerator.MoveNext() ) {
+           if (myEnumerator.Current != 77.7)
+             throw new Exception("Repeat (2) test failed");
+        }
+      }
+#endif
     }
 
     {
@@ -516,6 +533,96 @@ public class li_std_vector_runme {
       li_std_vector.halve_in_place(dvec);
     }
 
+    // Dispose()
+    {
+      using (StructVector vs = new StructVector() { new Struct(0.0), new Struct(11.1) } )
+      using (DoubleVector vd = new DoubleVector() { 0.0, 11.1 } ) {
+      }
+    }
+
+    // More wrapped methods
+    {
+      RealVector v0 = li_std_vector.vecreal(new RealVector());
+      float flo = 123.456f;
+      v0.Add(flo);
+      flo = v0[0];
+
+      IntVector v1 = li_std_vector.vecintptr(new IntVector());
+      IntPtrVector v2 = li_std_vector.vecintptr(new IntPtrVector());
+      IntConstPtrVector v3 = li_std_vector.vecintconstptr(new IntConstPtrVector());
+
+      v1.Add(123);
+      v2.Clear();
+      v3.Clear();
+
+      StructVector v4 = li_std_vector.vecstruct(new StructVector());
+      StructPtrVector v5 = li_std_vector.vecstructptr(new StructPtrVector());
+      StructConstPtrVector v6 = li_std_vector.vecstructconstptr(new StructConstPtrVector());
+
+      v4.Add(new Struct(123));
+      v5.Add(new Struct(123));
+      v6.Add(new Struct(123));
+    }
+
+    // Test vectors of pointers
+    {
+      StructPtrVector inputvector = new StructPtrVector();
+      int arrayLen = 10;
+      for (int i=0; i<arrayLen; i++) {
+        inputvector.Add(new Struct(i/10.0));
+      }
+      Struct[] outputarray = new Struct[arrayLen];
+      inputvector.CopyTo(outputarray);
+      for(int i=0; i<arrayLen; i++) {
+        if (outputarray[i].num != inputvector[i].num)
+          throw new Exception("StructPtrVector test (1) failed, i:" + i);
+      }
+      foreach (Struct s in inputvector) {
+        s.num += 20.0;
+      }
+      for(int i=0; i<arrayLen; i++) {
+        if (outputarray[i].num != 20.0 + i/10.0)
+          throw new Exception("StructPtrVector test (2) failed (a deep copy was incorrectly made), i:" + i);
+      }
+
+      int rangeSize = 5;
+      int mid = arrayLen/2;
+      StructPtrVector returnedVec = inputvector.GetRange(mid, rangeSize);
+      for (int i=0; i<rangeSize; i++) {
+        if (inputvector[i+mid].num != returnedVec[i].num)
+          throw new Exception("StructPtrVector test (3) failed, i:" + i);
+      }
+    }
+
+    // Test vectors of const pointers
+    {
+      StructConstPtrVector inputvector = new StructConstPtrVector();
+      int arrayLen = 10;
+      for (int i=0; i<arrayLen; i++) {
+        inputvector.Add(new Struct(i/10.0));
+      }
+      Struct[] outputarray = new Struct[arrayLen];
+      inputvector.CopyTo(outputarray);
+      for(int i=0; i<arrayLen; i++) {
+        if (outputarray[i].num != inputvector[i].num)
+          throw new Exception("StructConstPtrVector test (1) failed, i:" + i);
+      }
+      foreach (Struct s in inputvector) {
+        s.num += 20.0;
+      }
+      for(int i=0; i<arrayLen; i++) {
+        if (outputarray[i].num != 20.0 + i/10.0)
+          throw new Exception("StructConstPtrVector test (2) failed (a deep copy was incorrectly made), i:" + i);
+      }
+
+      int rangeSize = 5;
+      int mid = arrayLen/2;
+      StructConstPtrVector returnedVec = inputvector.GetRange(mid, rangeSize);
+      for (int i=0; i<rangeSize; i++) {
+        if (inputvector[i+mid].num != returnedVec[i].num)
+          throw new Exception("StructConstPtrVector test (3) failed, i:" + i);
+      }
+    }
 
   }
 
