@@ -7,7 +7,7 @@
  * cffi language module for SWIG.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_cffi_cxx[] = "$Id: cffi.cxx 10265 2008-02-25 22:07:28Z wsfulton $";
+char cvsroot_cffi_cxx[] = "$Id: cffi.cxx 11136 2009-02-20 23:55:37Z wsfulton $";
 
 #include "swigmod.h"
 #include "cparse.h"
@@ -22,7 +22,8 @@ public:
   String *f_clhead;
   String *f_clwrap;
   bool CWrap;     // generate wrapper file for C code?  
-  File *f_cxx;
+  File *f_begin;
+  File *f_runtime;
   File *f_cxx_header;
   File *f_cxx_wrapper;
   File *f_clos;
@@ -67,6 +68,7 @@ private:
 void CFFI::main(int argc, char *argv[]) {
   int i;
 
+  Preprocessor_define("SWIGCFFI 1", 0);
   SWIG_library_directory("cffi");
   SWIG_config_file("cffi.swg");
   generate_typedef_flag = 0;
@@ -119,16 +121,15 @@ int CFFI::top(Node *n) {
 
   Printf(lisp_filename, "%s%s.lisp", SWIG_output_directory(), module);
 
-  File *f_lisp = NewFile(lisp_filename, "w");
-  NewFile(lisp_filename, "w");
+  File *f_lisp = NewFile(lisp_filename, "w", SWIG_output_files());
   if (!f_lisp) {
     FileErrorDisplay(lisp_filename);
     SWIG_exit(EXIT_FAILURE);
   }
 
   if (CPlusPlus || CWrap) {
-    f_cxx = NewFile(cxx_filename, "w");
-    if (!f_cxx) {
+    f_begin = NewFile(cxx_filename, "w", SWIG_output_files());
+    if (!f_begin) {
       Close(f_lisp);
       Delete(f_lisp);
       Printf(stderr, "Unable to open %s for writing\n", cxx_filename);
@@ -137,7 +138,7 @@ int CFFI::top(Node *n) {
 
     String *clos_filename = NewString("");
     Printf(clos_filename, "%s%s-clos.lisp", SWIG_output_directory(), module);
-    f_clos = NewFile(clos_filename, "w");
+    f_clos = NewFile(clos_filename, "w", SWIG_output_files());
     if (!f_clos) {
       Close(f_lisp);
       Delete(f_lisp);
@@ -145,22 +146,32 @@ int CFFI::top(Node *n) {
       SWIG_exit(EXIT_FAILURE);
     }
   } else {
-    f_cxx = NewString("");
+    f_begin = NewString("");
     f_clos = NewString("");
   }
 
-  f_cxx_header = f_cxx;
+  f_runtime = NewString("");
+  f_cxx_header = f_runtime;
   f_cxx_wrapper = NewString("");
 
   Swig_register_filebyname("header", f_cxx_header);
   Swig_register_filebyname("wrapper", f_cxx_wrapper);
-  Swig_register_filebyname("runtime", f_cxx);
+  Swig_register_filebyname("begin", f_begin);
+  Swig_register_filebyname("runtime", f_runtime);
   Swig_register_filebyname("lisphead", f_clhead);
   if (!no_swig_lisp)
     Swig_register_filebyname("swiglisp", f_cl);
   else
     Swig_register_filebyname("swiglisp", f_null);
 
+  Swig_banner(f_begin);
+
+  Printf(f_runtime, "\n");
+  Printf(f_runtime, "#define SWIGCFFI\n");
+  Printf(f_runtime, "\n");
+
+  Swig_banner_target_lang(f_lisp, ";;;");
+
   Language::top(n);
   Printf(f_lisp, "%s\n", f_clhead);
   Printf(f_lisp, "%s\n", f_cl);
@@ -171,8 +182,10 @@ int CFFI::top(Node *n) {
   Delete(f_cl);
   Delete(f_clhead);
   Delete(f_clwrap);
-  Close(f_cxx);
-  Delete(f_cxx);
+  Dump(f_runtime, f_begin);
+  Close(f_begin);
+  Delete(f_runtime);
+  Delete(f_begin);
   Delete(f_cxx_wrapper);
   Delete(f_null);
 
@@ -232,12 +245,12 @@ void CFFI::emit_defmethod(Node *n) {
 
   ParmList *pl = Getattr(n, "parms");
   int argnum = 0;
-  Node *parent = parentNode(n);
+  Node *parent = getCurrentClass();
   bool first = 0;
   
   for (Parm *p = pl; p; p = nextSibling(p), argnum++) {
     String *argname = Getattr(p, "name");
-    String *ffitype = Swig_typemap_lookup_new("lispclass", p, "", 0);
+    String *ffitype = Swig_typemap_lookup("lispclass", p, "", 0);
 
     int tempargname = 0;
 
@@ -287,11 +300,11 @@ void CFFI::emit_initialize_instance(Node *n) {
 
   ParmList *pl = Getattr(n, "parms");
   int argnum = 0;
-  Node *parent = parentNode(n);
+  Node *parent = getCurrentClass();
 
   for (Parm *p = pl; p; p = nextSibling(p), argnum++) {
     String *argname = Getattr(p, "name");
-    String *ffitype = Swig_typemap_lookup_new("lispclass", p, "", 0);
+    String *ffitype = Swig_typemap_lookup("lispclass", p, "", 0);
 
     int tempargname = 0;
     if (!argname) {
@@ -324,18 +337,18 @@ void CFFI::emit_initialize_instance(Node *n) {
 }
 
 void CFFI::emit_setter(Node *n) {
-  Node *p = parentNode(n);
+  Node *parent = getCurrentClass();
   Printf(f_clos, "(cl:defmethod (cl:setf %s) (arg0 (obj %s))\n  (%s (ff-pointer obj) arg0))\n\n",
          lispify_name(n, Getattr(n, "name"), "'method"),
-         lispify_name(p, lispy_name(Char(Getattr(p, "sym:name"))), "'class"), lispify_name(n, Getattr(n, "sym:name"), "'function"));
+         lispify_name(parent, lispy_name(Char(Getattr(parent, "sym:name"))), "'class"), lispify_name(n, Getattr(n, "sym:name"), "'function"));
 }
 
 
 void CFFI::emit_getter(Node *n) {
-  Node *p = parentNode(n);
+  Node *parent = getCurrentClass();
   Printf(f_clos, "(cl:defmethod %s ((obj %s))\n  (%s (ff-pointer obj)))\n\n",
          lispify_name(n, Getattr(n, "name"), "'method"),
-         lispify_name(p, lispy_name(Char(Getattr(p, "sym:name"))), "'class"), lispify_name(n, Getattr(n, "sym:name"), "'function"));
+         lispify_name(parent, lispy_name(Char(Getattr(parent, "sym:name"))), "'class"), lispify_name(n, Getattr(n, "sym:name"), "'function"));
 }
 
 int CFFI::memberfunctionHandler(Node *n) {
@@ -354,9 +367,9 @@ int CFFI::functionWrapper(Node *n) {
 
   ParmList *parms = Getattr(n, "parms");
   String *iname = Getattr(n, "sym:name");
-  Wrapper *wrap = NewWrapper();
+  Wrapper *f = NewWrapper();
 
-  String *raw_return_type = Swig_typemap_lookup_new("ctype", n, "", 0);
+  String *raw_return_type = Swig_typemap_lookup("ctype", n, "", 0);
   SwigType *return_type = Swig_cparse_type(raw_return_type);
   SwigType *resolved = SwigType_typedef_resolve_all(return_type);
   int is_void_return = (Cmp(resolved, "void") == 0);
@@ -364,7 +377,7 @@ int CFFI::functionWrapper(Node *n) {
 
   if (!is_void_return) {
     String *lresult_init = NewStringf("lresult = (%s)0", raw_return_type);
-    Wrapper_add_localv(wrap, "lresult", raw_return_type, lresult_init, NIL);
+    Wrapper_add_localv(f, "lresult", raw_return_type, lresult_init, NIL);
     Delete(lresult_init);
   }
 
@@ -373,7 +386,7 @@ int CFFI::functionWrapper(Node *n) {
     overname = Getattr(n, "sym:overname");
   } else {
     if (!addSymbol(iname, n)) {
-      DelWrapper(wrap);
+      DelWrapper(f);
       return SWIG_ERROR;
     }
   }
@@ -385,11 +398,11 @@ int CFFI::functionWrapper(Node *n) {
   Setattr(n, "wrap:name", wname);
 
   // Emit all of the local variables for holding arguments.
-  emit_args(Getattr(n, "type"), parms, wrap);
+  emit_parameter_variables(parms, f);
 
   // Attach the standard typemaps 
-  Swig_typemap_attach_parms("ctype", parms, wrap);
-  emit_attach_parmmaps(parms, wrap);
+  Swig_typemap_attach_parms("ctype", parms, f);
+  emit_attach_parmmaps(parms, f);
 
   int num_arguments = emit_num_arguments(parms);
   String *name_and_parms = NewStringf("%s (", wname);
@@ -426,7 +439,7 @@ int CFFI::functionWrapper(Node *n) {
     {
       Replaceall(parm_code, "$input", arg);
       Setattr(p, "emit:input", arg);
-      Printf(wrap->code, "%s\n", parm_code);
+      Printf(f->code, "%s\n", parm_code);
       p = Getattr(p, "tmap:in:next");
     }
 
@@ -436,25 +449,26 @@ int CFFI::functionWrapper(Node *n) {
 
   // Emit the function definition
   String *signature = SwigType_str(return_type, name_and_parms);
-  Printf(wrap->def, "EXPORT %s {", signature);
-  Printf(wrap->code, "  try {\n");
-  emit_action(n, wrap);
-  if (!is_void_return) {
-    String *result_convert = Swig_typemap_lookup_new("out", n, "result", 0);
-    Replaceall(result_convert, "$result", "lresult");
-    Printf(wrap->code, "%s\n", result_convert);
-    Printf(wrap->code, "    return lresult;\n");
-    Delete(result_convert);
-  }
+  Printf(f->def, "EXPORT %s {", signature);
+  Printf(f->code, "  try {\n");
 
-  Printf(wrap->code, "  } catch (...) {\n");
+  String *actioncode = emit_action(n);
+
+  String *result_convert = Swig_typemap_lookup_out("out", n, "result", f, actioncode);
+  Replaceall(result_convert, "$result", "lresult");
+  Printf(f->code, "%s\n", result_convert);
+  if(!is_void_return) Printf(f->code, "    return lresult;\n");
+  Delete(result_convert);
+  emit_return_variable(n, Getattr(n, "type"), f);
+
+  Printf(f->code, "  } catch (...) {\n");
   if (!is_void_return)
-    Printf(wrap->code, "    return (%s)0;\n", raw_return_type);
-  Printf(wrap->code, "  }\n");
-  Printf(wrap->code, "}\n");
+    Printf(f->code, "    return (%s)0;\n", raw_return_type);
+  Printf(f->code, "  }\n");
+  Printf(f->code, "}\n");
 
   if (CPlusPlus)
-    Wrapper_print(wrap, f_cxx);
+    Wrapper_print(f, f_runtime);
 
   if (CPlusPlus) {
     emit_defun(n, wname);
@@ -482,7 +496,7 @@ int CFFI::functionWrapper(Node *n) {
   //   }
 
   Delete(wname);
-  DelWrapper(wrap);
+  DelWrapper(f);
 
   return SWIG_OK;
 }
@@ -505,7 +519,7 @@ void CFFI::emit_defun(Node *n, String *name) {
   emit_inline(n, func_name);
 
   Printf(f_cl, "\n(cffi:defcfun (\"%s\" %s)", name, func_name);
-  String *ffitype = Swig_typemap_lookup_new("cout", n, ":pointer", 0);
+  String *ffitype = Swig_typemap_lookup("cout", n, ":pointer", 0);
 
   Printf(f_cl, " %s", ffitype);
   Delete(ffitype);
@@ -519,7 +533,7 @@ void CFFI::emit_defun(Node *n, String *name) {
 
     String *argname = Getattr(p, "name");
 
-    ffitype = Swig_typemap_lookup_new("cin", p, "", 0);
+    ffitype = Swig_typemap_lookup("cin", p, "", 0);
 
     int tempargname = 0;
     if (!argname) {
@@ -567,7 +581,7 @@ int CFFI::variableWrapper(Node *n) {
   //    return SWIG_OK;
 
   String *var_name = Getattr(n, "sym:name");
-  String *lisp_type = Swig_typemap_lookup_new("cin", n, "", 0);
+  String *lisp_type = Swig_typemap_lookup("cin", n, "", 0);
   String *lisp_name = lispify_name(n, var_name, "'variable");
 
   if (Strcmp(lisp_name, "t") == 0 || Strcmp(lisp_name, "T") == 0)
@@ -584,7 +598,7 @@ int CFFI::variableWrapper(Node *n) {
 int CFFI::typedefHandler(Node *n) {
   if (generate_typedef_flag && strncmp(Char(Getattr(n, "type")), "enum", 4)) {
     String *lisp_name = lispify_name(n, Getattr(n, "name"), "'typename");
-    Printf(f_cl, "\n(cffi:defctype %s %s)\n", lisp_name, Swig_typemap_lookup_new("cin", n, "", 0));
+    Printf(f_cl, "\n(cffi:defctype %s %s)\n", lisp_name, Swig_typemap_lookup("cin", n, "", 0));
     emit_export(n, lisp_name);
   }
   return Language::typedefHandler(n);
@@ -629,7 +643,7 @@ int CFFI::enumDeclaration(Node *n) {
     else {
       String *type = Getattr(c, "type");
       String *converted_value = convert_literal(value, type);
-      Printf(f_cl, "\n\t(%s %s)", slot_name, converted_value);
+      Printf(f_cl, "\n\t(%s #.%s)", slot_name, converted_value);
       Delete(converted_value);
     }
     Delete(value);
@@ -672,7 +686,7 @@ void CFFI::emit_class(Node *n) {
 
   Printf(supers, ")");
   Printf(f_clos, "\n(cl:defclass %s%s", lisp_name, supers);
-  Printf(f_clos, "\n  ((ff :reader ff-pointer)))\n\n");
+  Printf(f_clos, "\n  ((ff-pointer :reader ff-pointer)))\n\n");
 
   Parm *pattern = NewParm(Getattr(n, "name"), NULL);
 
