@@ -11,7 +11,7 @@
  * Perl5 language module for SWIG.
  * ------------------------------------------------------------------------- */
 
-char cvsroot_perl5_cxx[] = "$Id: perl5.cxx 10453 2008-05-15 21:18:44Z wsfulton $";
+char cvsroot_perl5_cxx[] = "$Id: perl5.cxx 11133 2009-02-20 07:52:24Z wsfulton $";
 
 #include "swigmod.h"
 #include "cparse.h"
@@ -78,6 +78,7 @@ static String *command_tab = 0;
 static String *constant_tab = 0;
 static String *variable_tab = 0;
 
+static File *f_begin = 0;
 static File *f_runtime = 0;
 static File *f_header = 0;
 static File *f_wrappers = 0;
@@ -207,6 +208,7 @@ public:
     }
 
     Preprocessor_define("SWIGPERL 1", 0);
+    // SWIGPERL5 is deprecated, and no longer documented.
     Preprocessor_define("SWIGPERL5 1", 0);
     SWIG_typemap_lang("perl5");
     SWIG_config_file("perl5.swg");
@@ -222,11 +224,12 @@ public:
     /* Initialize all of the output files */
     String *outfile = Getattr(n, "outfile");
 
-    f_runtime = NewFile(outfile, "w");
-    if (!f_runtime) {
+    f_begin = NewFile(outfile, "w", SWIG_output_files());
+    if (!f_begin) {
       FileErrorDisplay(outfile);
       SWIG_exit(EXIT_FAILURE);
     }
+    f_runtime = NewString("");
     f_init = NewString("");
     f_header = NewString("");
     f_wrappers = NewString("");
@@ -234,6 +237,7 @@ public:
     /* Register file targets with the SWIG file handler */
     Swig_register_filebyname("header", f_header);
     Swig_register_filebyname("wrapper", f_wrappers);
+    Swig_register_filebyname("begin", f_begin);
     Swig_register_filebyname("runtime", f_runtime);
     Swig_register_filebyname("init", f_init);
 
@@ -252,11 +256,12 @@ public:
     constant_tab = NewString("static swig_constant_info swig_constants[] = {\n");
     variable_tab = NewString("static swig_variable_info swig_variables[] = {\n");
 
-    Swig_banner(f_runtime);
+    Swig_banner(f_begin);
 
+    Printf(f_runtime, "\n");
     Printf(f_runtime, "#define SWIGPERL\n");
     Printf(f_runtime, "#define SWIG_CASTRANK_MODE\n");
-
+    Printf(f_runtime, "\n");
 
     // Is the imported module in another package?  (IOW, does it use the
     // %module(package="name") option and it's different than the package
@@ -315,7 +320,7 @@ public:
 	pmfile = NewStringf("%s.pm", m);
       }
       String *filen = NewStringf("%s%s", SWIG_output_directory(), pmfile);
-      if ((f_pm = NewFile(filen, "w")) == 0) {
+      if ((f_pm = NewFile(filen, "w", SWIG_output_files())) == 0) {
 	FileErrorDisplay(filen);
 	SWIG_exit(EXIT_FAILURE);
       }
@@ -332,10 +337,7 @@ public:
       Delete(boot_name);
     }
 
-    Printf(f_pm, "# This file was automatically generated by SWIG (http://www.swig.org).\n");
-    Printf(f_pm, "# Version %s\n", Swig_package_version());
-    Printf(f_pm, "#\n");
-    Printf(f_pm, "# Don't modify this file, modify the SWIG interface instead.\n");
+    Swig_banner_target_lang(f_pm, "#");
     Printf(f_pm, "\n");
 
     Printf(f_pm, "package %s;\n", module);
@@ -524,14 +526,16 @@ public:
     Delete(underscore_module);
 
     /* Close all of the files */
-    Dump(f_header, f_runtime);
-    Dump(f_wrappers, f_runtime);
-    Wrapper_pretty_print(f_init, f_runtime);
+    Dump(f_runtime, f_begin);
+    Dump(f_header, f_begin);
+    Dump(f_wrappers, f_begin);
+    Wrapper_pretty_print(f_init, f_begin);
     Delete(f_header);
     Delete(f_wrappers);
     Delete(f_init);
-    Close(f_runtime);
+    Close(f_begin);
     Delete(f_runtime);
+    Delete(f_begin);
     return SWIG_OK;
   }
 
@@ -1141,8 +1145,9 @@ public:
 
     /* Do some work on the class name */
     if (verbose > 0) {
+      String *modulename = Getattr(clsmodule, "name");
       fprintf(stdout, "setclassname: Found sym:name: %s\n", Char(symname));
-      fprintf(stdout, "setclassname: Found module: %s\n", Char(clsmodule));
+      fprintf(stdout, "setclassname: Found module: %s\n", Char(modulename));
       fprintf(stdout, "setclassname: No package found\n");
     }
 
@@ -1624,7 +1629,7 @@ public:
 	} else if (Strcmp(code, "include") == 0) {
 	  /* Include a file into the .pm file */
 	  if (value) {
-	    FILE *f = Swig_open(value);
+	    FILE *f = Swig_include_open(value);
 	    if (!f) {
 	      Printf(stderr, "%s : Line %d. Unable to locate file %s\n", input_file, line_number, value);
 	    } else {
